import datetime as dt

import pandas as pd
import pytest

import aiostp


def test_create_order():
    orden = aiostp.Orden(
        monto=29.00,
        conceptoPago='prueba',
        cuentaBeneficiario='072691004495711499',
        nombreBeneficiario='Pancho Villa',
        institucionContraparte=90646,
        cuentaOrdenante='646180157075068123',
    )

    assert orden.monto == 29.00


def test_create_order_invalid_account_type():
    with pytest.raises(ValueError) as exc_info:
        aiostp.Orden(
            monto=29.00,
            conceptoPago='prueba',
            cuentaBeneficiario='072691004495711499',
            nombreBeneficiario='Pancho Villa',
            institucionContraparte=12345,
            cuentaOrdenante='646180157075068123',
        )
    assert '12345 no se corresponde a un banco' in str(exc_info.value)


def test_create_order_invalid_amount():
    with pytest.raises(ValueError) as exc_info:
        aiostp.Orden(
            monto='rr',
            conceptoPago='prueba',
            cuentaBeneficiario='072691004495711499',
            nombreBeneficiario='Pancho Villa',
            institucionContraparte=90646,
            cuentaOrdenante='646180157075068123',
        )
    assert 'value is not a valid float (type=type_error.float)' in str(
        exc_info.value
    )


@pytest.mark.asyncio
@pytest.mark.vcr
async def test_consulta_ordenes_enviadas():
    enviadas = await aiostp.Orden.consulta_enviadas()

    assert isinstance(enviadas, pd.DataFrame)
    assert len(enviadas) > 0


@pytest.mark.asyncio
@pytest.mark.vcr
async def test_consulta_ordenes_recibidas():
    recibidas = await aiostp.Orden.consulta_recibidas()

    assert isinstance(recibidas, pd.DataFrame)
    assert len(recibidas) > 0


@pytest.mark.asyncio
@pytest.mark.vcr
async def test_consulta():
    consulta = await aiostp.Orden.consulta()

    assert isinstance(consulta, pd.DataFrame)
    assert len(consulta) > 0


@pytest.mark.asyncio
@pytest.mark.vcr
async def test_consulta_ordenes_enviadas_con_fecha():
    enviadas = await aiostp.Orden.consulta_enviadas(dt.date(2020, 4, 20))

    assert isinstance(enviadas, pd.DataFrame)
    assert len(enviadas) > 0


@pytest.mark.asyncio
@pytest.mark.vcr
async def test_consulta_ordenes_enviadas_con_fecha_sin_resultados():
    enviadas = await aiostp.Orden.consulta_enviadas(dt.date(2021, 4, 20))

    assert isinstance(enviadas, pd.DataFrame)
    assert len(enviadas) == 0


@pytest.mark.asyncio
@pytest.mark.vcr
async def test_consulta_orden_enviada_clave_rastreo():
    fecha = dt.date(2021, 1, 14)
    enviada = await aiostp.Orden.consulta_clave_rastreo_enviada(
        'CR1610641277', fechaOperacion=fecha
    )

    assert enviada.claveRastreo == 'CR1610641277'



from unittest import mock

import pytest
from aiohttp.client_exceptions import ClientResponseError
from stpmex.exc import InvalidPassphrase, StpmexException

from aiostp.http.client import Session

CUENTA_ENDPOINT = '/cuentaModule/fisica'


class MockResponse:
    def __init__(self, json, status):
        self._json = json
        self.status = status

    async def json(self):
        return self._json

    async def __aexit__(self, exc_type, exc, tb):
        pass

    async def __aenter__(self):
        return self


@pytest.mark.asyncio
async def test_forbidden_without_vpn():
    session = Session()
    await session.configure(demo=False)
    with pytest.raises(ClientResponseError) as exc_info:
        await session.get('/application.wadl')
    assert exc_info.value.status == 403


@pytest.mark.asyncio
async def test_error():
    error = dict(descripcion='unknown code', id=999999)
    session = Session()
    await session.configure(demo=True)

    with mock.patch(
        'aiostp.http.client.aiohttp.ClientSession.request',
        return_value=MockResponse(error, 200),
    ):
        with pytest.raises(StpmexException) as exc_info:
            await session.post(CUENTA_ENDPOINT, data={})
        exc = exc_info.value

    assert type(exc) == StpmexException


@pytest.mark.asyncio
async def test_incorrect_passphrase():
    session = Session()
    with pytest.raises(InvalidPassphrase):
        await session.configure(
            priv_key='tests/data/pkey.pem', priv_key_passphrase='wrong'
        )


@pytest.mark.asyncio
async def test_configure():
    session = Session()
    await session.configure(
        empresa='EMPRESA PRUEBA', clabe='1234567890', pool_size=6
    )
    # running configure again just to re-generate aiohttp session
    await session.configure()

    assert session.empresa == 'EMPRESA PRUEBA'
    assert session.clabe == '1234567890'
    assert session._connection_pool_size == 6



import os
from typing import Any, Dict, List, Optional, Union

import aiohttp
from aiohttp.client_reqrep import ClientResponse
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey
from stpmex.client import _raise_description_error_exc, _raise_description_exc

from ..auth import gen_pkey
from ..version import __version__ as client_version

DEMO_HOST = 'https://demo.stpmex.com:7024'
PROD_HOST = 'https://prod.stpmex.com'


class Session:
    base_url: str
    soap_url: str
    empresa: str
    pkey: Optional[RSAPrivateKey]

    _connection_pool_size: int
    _ssl_verify: bool

    def __init__(self):
        self.empresa = os.getenv('EMPRESA', '')
        self.clabe = os.getenv('CLABE', '')

        self._ssl_verify = True
        self._connection_pool_size = 4
        self.pkey = None

        self.base_url = f'{PROD_HOST}/speiws/rest'
        self.soap_url = f'{PROD_HOST}/spei/webservices/SpeiConsultaServices'

    async def configure(
        self,
        demo: bool = False,
        priv_key: Union[str, bytes] = '',
        priv_key_passphrase: str = '',
        empresa: str = '',
        clabe: str = '',
        pool_size: Optional[int] = None,
    ):
        priv_key = priv_key or os.getenv('STP_PRIVATE_LOCATION', '')
        priv_key_passphrase = priv_key_passphrase or os.getenv(
            'STP_KEY_PASSPHRASE', ''
        )
        self.pkey = await gen_pkey(priv_key, priv_key_passphrase)

        if demo:
            host_url = DEMO_HOST
            self._ssl_verify = False
        else:
            host_url = PROD_HOST
            self._ssl_verify = True
        self.base_url = f'{host_url}/speiws/rest'
        self.soap_url = f'{host_url}/spei/webservices/SpeiConsultaServices'

        if empresa:
            self.empresa = empresa

        if clabe:
            self.clabe = clabe

        if pool_size:
            self._connection_pool_size = pool_size

    async def get(
        self,
        endpoint: str,
    ) -> Union[Dict[str, Any], List[Any]]:
        return await self.request('GET', endpoint, {})

    async def post(
        self, endpoint: str, data: Dict[str, Any]
    ) -> Union[Dict[str, Any], List[Any]]:
        return await self.request('POST', endpoint, data)

    async def request(
        self,
        method: str,
        endpoint: str,
        data: Dict[str, Any],
        **kwargs: Any,
    ) -> Union[Dict[str, Any], List[Any]]:
        url = self.base_url + endpoint
        conn = aiohttp.TCPConnector(
            limit=self._connection_pool_size, ssl=self._ssl_verify
        )
        async with aiohttp.ClientSession(
            connector=conn,
            headers={'User-Agent': f'stpmex-python/{client_version}'},
        ) as session:
            async with session.request(
                method,
                url,
                json=data,
                **kwargs,
            ) as response:
                await self._check_response(response)
                resultado = await response.json()
        if 'resultado' in resultado:  # Some responses are enveloped
            resultado = resultado['resultado']
        return resultado

    async def soap_request(
        self,
        method: str,
        endpoint: str,
        data: str,
        **kwargs: Any,
    ):
        url = self.soap_url + endpoint
        conn = aiohttp.TCPConnector(
            limit=self._connection_pool_size, ssl=self._ssl_verify
        )
        async with aiohttp.ClientSession(
            connector=conn,
            headers={'User-Agent': f'stpmex-python/{client_version}'},
        ) as session:
            async with session.request(method, url, json=data) as response:
                if response.status != 200:
                    response.raise_for_status()
                resultado = await response.text()
        return resultado

    @staticmethod
    async def _check_response(response: ClientResponse) -> None:
        if response.status != 200:
            response.raise_for_status()
        resp = await response.json()
        if isinstance(resp, dict):
            try:
                _raise_description_error_exc(resp)
            except KeyError:
                ...
            try:
                assert resp['descripcion']
                _raise_description_exc(resp)
            except (AssertionError, KeyError):
                ...
        response.raise_for_status()
